module Menu
# = Clase Menu
# Sirve para instanciar Objetos tipo Menu.
class Menu
    include Comparable

    attr_reader :titulo , :porcentaje, :platos, :vct, :proteinas, :grasas, :hidratos
    
    #==Constructor
#===Pasamos los parametros para construir un objeto menú:
# - Título: Título del menú.
# - Porcentaje: Porcentaje en Kcal.
# - Platos: Array de platos que contiene el menú.
# - Vct: Valor Calórico total.
# - Porcentaje de proteinas de la dieta.
# - Porcentaje de grasas.
# - Porcentaje de hidratos.

    def initialize (titulo, porcentaje, platos, vct, proteinas, grasas, hidratos)
        @titulo = titulo # - Título: Título del menú.
        @porcentaje = porcentaje # - Porcentaje: Porcentaje en Kcal.
        @platos = Array.new # - Platos: Array de platos que contiene el menú.
        platos.each{|x| @platos.push(Plato.new(x))}
        @vct = vct # - Vct: Valor Calórico total.
        @proteinas = proteinas # - Porcentaje de proteinas de la dieta.
        @grasas = grasas # - Porcentaje de grasas.
        @hidratos = hidratos # - Porcentaje de hidratos.
    end
    
    def <=>(anOther)
        return nil if anOther == nil
        @vct <=> anOther.getVct
        #x = [@vct, @proteinas, @grasas, @hidratos]
        #y = [anOther.getVct, anOther.getProteinas, anOther.getGrasas, anOther.getHidratos]
        #x <=> y
    end
    
    def ==(anOther)
        return nil if anOther == nil
        x = [@vct, @proteinas, @grasas, @hidratos]
        y = [anOther.getVct, anOther.getProteinas, anOther.getGrasas, anOther.getHidratos]
        x == y
    end
    
    #* Devuelve un string con el Menú completo correctamente formateado
    def to_s
       
        porcentaje = corregir_porcentaje
        
        @titulo + " (#{porcentaje}%)\n" + getConjuntoPlatos + "V.C.T. | % #{getVct} kcal | #{getProteinas}% - #{getGrasas}% - #{getHidratos}%\n"
    end
    
    private
    def corregir_porcentaje
        t = ""
        @porcentaje.each{|x| t = t + " - " + x.to_s}
        final = t[3,t.length]
        final
    end
    public
    #* Devuelve el plato del índice indicado en el parámetro.    
    def getPlato(index)
        @platos[index].to_s
    end

    #* Devuelve el título del Menú.
    def getTitulo
        @titulo
    end
       
    #* Devuelve el porcentaje de ls Ingesta Diaria recomendada.
    def getIngesta
        @porcentaje 
    end
    #* Devuelve el Valor calórico total del Menú.
    def getVct
        @vct 
    end
    
    #* Devuelve el porcentaje de proteínas del Menú.
    def getProteinas
        @proteinas 
    end
    
    #* Devuelve el porcentaje de grasas del Menú.
    def getGrasas
        @grasas 
    end
    
    #* Devuelve el porcentaje de hidratos de carbono del Menú.
    def getHidratos
        @hidratos 
    end
    #* Devuelve la descripción de un plato del Menú.
    def getDescripcion (x)
        @platos[x].descripcion
    end
    #* Devuelve un string en formato lista con los platos del Menú.
    def getConjuntoPlatos
        t = ""
        @platos.each{|x| t = t + x.to_s + "\n"}
        t
    end
end
end

# = Clase Alimentos
# La clase alimentos hereda de la clase Menu para poder organizar los Menus por grupos de alimentos.
class Alimentos < Menu::Menu
    
    attr_reader :grupo
#==Constructor
#Pasamos los parametros para construir un objeto menú y el parámetro grupo para poder clasificarlo e instanciar un objeto de la clase Alimentos.    
    def initialize (titulo, porcentaje, platos, vct, proteinas, grasas, hidratos, grupo)
        super(titulo, porcentaje, platos, vct, proteinas, grasas, hidratos)
        @grupo = grupo
    end
    
    def to_s
        s = ""
        s << super.to_s
        s << "Grupo: #{@grupo}\n"
        s
    end
    
    
end

# = Clase Edades
# La clase edades hereda de la clase Menu para poder organizar los Menus por grupos de edades.

class Edades < Menu::Menu
    
    attr_reader :grupo
    
#==Constructor
#Pasamos los parametros para construir un objeto menú y el parámetro grupo para poder clasificarlo e instanciar un objeto de la clase Edades.
    def initialize (titulo, porcentaje, platos, vct, proteinas, grasas, hidratos, grupo)
        super(titulo, porcentaje, platos, vct, proteinas, grasas, hidratos)
        @grupo = grupo
    end
    
    def to_s
        s = ""
        s << super.to_s
        s << "Grupo: #{@grupo}\n"
        s
    end
    
end

class MenuDSL 
     
    def initialize(&bloque)
        @titulo
        @porcentaje
        @platos = []
        @vct
        @proteinas
        @grasas
        @hidratos
        @grupo
        @edades
        
        if block_given?  
            if bloque.arity == 1
                yield self
            else
                instance_eval(&bloque) 
            end
        end
    end
    
    def titulo (name, options = {})
        @titulo = name
        @grupo = "#{options[:grupo]}" if options[:grupo]
        @edades = "#{options[:edad]}" if options[:edad]
    end
    
    def ingesta (options = {})
        porcentaje = []
        porcentaje << "#{options[:min]}" if options[:min]
        porcentaje << "#{options[:max]}" if options[:max]
        @porcentaje = porcentaje
    end
    
    def plato (options = {})
        plato = []
        plato << "#{options[:descripcion]}" if options[:descripcion]
        plato << "#{options[:porcion]}" if options[:porcion]
        plato << "#{options[:gramos]}" if options[:gramos]
        @platos.push(Plato.new(plato))
    end
    
    def porcentajes (options = {})
        @vct = "#{options[:vct]}" if options[:vct]
        @proteinas = "#{options[:proteinas]}" if options[:proteinas]
        @grasas = "#{options[:grasas]}" if options[:grasas]
        @hidratos = "#{options[:hidratos]}" if options[:hidratos]
    end
    
    
    
    def to_s
       
        porcentaje = corregir_porcentaje
        
        @titulo + " (#{porcentaje}%)\n" + getConjuntoPlatos + "V.C.T. | % #{getVct} kcal | #{getProteinas}% - #{getGrasas}% - #{getHidratos}%\n Grupo de alimentos: #{getGrupo}\n Grupo de edad: #{getEdades}\n"
    end
    
# METODOS GET ###################################################################################################################################    
     #* Devuelve el plato del índice indicado en el parámetro.    
    def getPlato(index)
        @platos[index].to_s
    end

    #* Devuelve el título del Menú.
    def getTitulo
        @titulo
    end
       
    #* Devuelve el porcentaje de ls Ingesta Diaria recomendada.
    def getIngesta
        @porcentaje 
    end
    #* Devuelve el Valor calórico total del Menú.
    def getVct
        @vct 
    end
    
    #* Devuelve el porcentaje de proteínas del Menú.
    def getProteinas
        @proteinas 
    end
    
    #* Devuelve el porcentaje de grasas del Menú.
    def getGrasas
        @grasas 
    end
    
    #* Devuelve el porcentaje de hidratos de carbono del Menú.
    def getHidratos
        @hidratos 
    end
    
    #* Devuelve la descripción de un plato del Menú.
    def getDescripcion (x)
        @platos[x].descripcion
    end
    
    #* Devuelve un string en formato lista con los platos del Menú.
    def getConjuntoPlatos
        t = ""
        @platos.each{|x| t = t + x.to_s + "\n"}
        t
    end
    
    #* Devuelve el Grupo de alimentos al que pertenece el Menú.
    def getGrupo
        @grupo 
    end
    
    #* Devuelve el Grupo de edad al que pertenece el Menú.
    def getEdades
       @edades 
    end
#################################################################################################################################################    

    private
    def corregir_porcentaje
        t = ""
        @porcentaje.each{|x| t = t + " - " + x.to_s}
        final = t[3,t.length]
        final
    end
end